unit AutorunPlugin;

interface

uses Windows;

type
  // Common function prototypes
  TPluginProc = function(Data: Pointer): integer; stdcall;
  TApiProc = function(FuncIndex: integer; Data: Pointer): integer; stdcall;

  // record for Init function
  PAutorunInterface = ^TAutorunInterface;
  TAutorunInterface = record
    nSize: integer;
    hWnd: HWND;
    pAdapter: Pointer;
  end;

  // parameters for plugin functions
  PAutorunFuncParams = ^TAutorunFuncParams;
  TAutorunFuncParams = record
    nSize: integer;
    Param: PWideChar;
    Value: PWideChar;
    MaxLen: integer;
  end;

  // function definition
  PPluginFuncInfo = ^TPluginFuncInfo;
  TPluginFuncInfo = record
    nSize: integer;
    Flags: integer;
    FuncName: PWideChar;
    FuncAddr: Pointer;
  end;

  // variable definition
  PVarInfo = ^TVarInfo;
  TVarInfo = record
    nSize: integer;
    Flags: integer;
    VarName: PWideChar;
    VarValue: PWideChar;
  end;

  // common record for API functions
  PGetData = ^TGetData;
  TGetData = record
    Text: PWideChar;
    Buffer: PWideChar;
  end;

  PExpandVars = ^TExpandVars;
  TExpandVars = TGetData;

  PParseNextData = ^TParseNextData;
  TParseNextData = TGetData;

const
  // buffer size for functions that receive buffers
  MAX_BUFFER_SIZE = 4096;

  // Function return values (error codes)
  AU_RESULT_OK = 0;
  AU_RESULT_ERR_UNKNOWN = 1;
  AU_RESULT_VARNOEXIST = 2;
  AU_RESULT_NOREDEFINECONST = 3;

  // Parser function return values
  AU_PARSER_NOMOREPARAMS = 0;
  AU_PARSER_REGULARPARAM = 1;
  AU_PARSER_NAMEDPARARAM = 2;

  // Flags for set variables proc
  AU_VAR_CONSTANT = 1;

  // Flags for set functions proc
  AU_FUNC_RETURNVALUE = 1;
  AU_FUNC_BOOLEAN =     2;

  // Indexes of API functions
  AU_PROC_ADDFUNC     = 1;
  AU_PROC_PARSER_NEXT = 2;
  AU_PROC_GETVAR      = 3;
  AU_PROC_SETVAR      = 4;
  AU_PROC_EXPANDVARS  = 5;

procedure AddFunc(AName: PWideChar; AAddr: Pointer; AFlags: integer);
function ParseNext(AParamStr: PWideChar; ABuffer: PWideChar): integer;
function GetVar(AVarName: PWideChar; Buffer: PWideChar): integer;
function SetVar(const AName, AValue: PWideChar; AFlags: integer): integer;
function ExpandVars(const AText: PWideChar; ABuffer: PWideChar): integer;

var
  Adapter: Pointer;

implementation

procedure AddFunc(AName: PWideChar; AAddr: Pointer; AFlags: integer);
var
  FuncInfo: TPluginFuncInfo;
begin
  with FuncInfo do
  begin
    nSize    := SizeOf(TPluginFuncInfo);
    Flags    := AFlags;
    FuncName := AName;
    FuncAddr := AAddr;
  end;
  TAPIProc(Adapter)(AU_PROC_ADDFUNC, @FuncInfo);
end;

function ParseNext(AParamStr: PWideChar; ABuffer: PWideChar): integer;
var
  Data: TParseNextData;
begin
  with Data do
  begin
    Text   := AParamStr;
    Buffer := ABuffer;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_PARSER_NEXT, @Data);
end;

function GetVar(AVarName: PWideChar; Buffer: PWideChar): integer;
var
  Data: TVarInfo;
begin
  with Data do
  begin
    nSize    := SizeOf(TVarInfo);
    VarName  := AVarName;
    VarValue := Buffer;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_GETVAR, @Data);
end;

function SetVar(const AName, AValue: PWideChar; AFlags: integer): integer;
var
  VarInfo: TVarInfo;
begin
  with VarInfo do
  begin
    nSize    := SizeOf(TVarInfo);
    Flags    := AFlags;
    VarName  := AName;
    VarValue := AValue;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_SETVAR, @VarInfo);
end;


function ExpandVars(const AText: PWideChar; ABuffer: PWideChar): integer;
var
  Data: TExpandVars;
begin
  with Data do
  begin
    Text   := AText;
    Buffer := ABuffer;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_EXPANDVARS, @Data);
end;


end.
